function [part,options]=MCA2_SIMPLE(img,dict,pars1,pars2,pars3,itermax,gamma,comptv,expdecrease,stop,mask,sigma)
%Morphological Component Analysis of a 2D image using highly redundant
%dictrionaries and sparsity promoting penalties. This function is
%simplified from the MCALAB function 'MCA2_Bcr'.
%Usage:
%   part = MCA2_SIMPLE(img,dict,pars1,pars2,pars3,itermax,gamma,expdecrease,stop,mask)
%Inputs:
%   img:        2D image matrix, n*n, if n ~= 2^J, use zero-padding and crop
%   at the end.
%   dict:   Names of dictionaries for each part.
%   parsi:  Parameters of dictionary.
%   itermax:    Number of relaxation iterations.
%   gamma:  TV regularization parameter.
%   comptv: Component to which TV regularization is applied.
%   expdecrease:    Exponential/Linear decrease of the regularization
%   parameter.
%   stop:   Stop criterion, the algorithm stops when lambda <= stop*sigma
%   (typically k=3), sigma is the noise WGN std.
%Outputs:
%   parti:  Estimated i-th semantic component (n*n image)
%   options:    Structure containing all options and parameters of the alogrithm, including default choice.

% Initializations.
global E;
	[N,M]  	      = size(img);
	n      	      = 2^(nextpow2(max(N,M)));
	imgpad 	      = zeros(n,n);
	maskpad	      = ones(n,n);
	imgpad(1:N,1:M)  = img;

    [n,J]  	      = quadlength(imgpad);
	thdtype       = 'Soft';
	options.thdtype = ['Threshold type: ' thdtype];

% Dictionary metadata.
	numberofdicts = LengthList(dict);
	options.nbcomp= sprintf('Number of morphological components: %d',numberofdicts);
	options.dict  = ['Transforms: [ ' dict ']'];
	str = 'Parameter 1 of transforms: [ ';
	for nb=1:numberofdicts, str = [str num2str(NthList(pars1,nb)) ' : ']; end
	options.pars1 = [str ']'];
	str = 'Parameter 2 of transforms: [ ';
	for nb=1:numberofdicts, str = [str num2str(NthList(pars2,nb)) ' : ']; end
	options.pars2 = [str ']'];	
	str = 'Parameter 3 of transforms: [ ';
	for nb=1:numberofdicts, str = [str num2str(NthList(pars3,nb)) ' : ']; end
	options.pars3 = [str ']'];
	part	      = zeros(n,n,numberofdicts);
    
% To estimate the WGN standard deviation using the MAD.
% The qmf is quite arbitrary, sufficiently regular to approach a good band-pass.
	if ~exist('sigma','var') | isempty(sigma),
	  qmf=MakeONFilter('Daubechies',4);
	  wc = FWT2_PO(imgpad,J-1,qmf);
	  hh = wc(n/2+1:n/2+floor(N/2),n/2+1:n/2+floor(M/2));hh = hh(:);
	  tmp   = maskpad(n/2+1:n/2+floor(N/2),n/2+1:n/2+floor(M/2));tmp = tmp(:);
	  sigma = MAD(hh(find(tmp)));
	  options.sigma = sprintf('Initial sigma estimated from data: %f',sigma);
	else
	  options.sigma = sprintf('Initial sigma fixed by the user: %f',sigma);
	end
	stopcriterion = stop*sigma;
	options.stopcriterion = sprintf('Stopping threshold: %f x sigma=%f',stop,stopcriterion);
	
% First pass: coeffs of the original image in each dictionary.
	coeff = FastLA2(imgpad,dict,pars1,pars2,pars3);

% Calculate the starting thd, which is the minimum of maximal coefficients 
% of the image in each dictionary.
	deltamax=StartingPoint(coeff,dict);
	delta=deltamax;
	options.lambdamax   = sprintf('Starting threshold: %f',deltamax);
	if expdecrease	
		lambda=(deltamax/stopcriterion)^(1/(1-itermax)); % Exponential decrease.
		options.lambdasched = sprintf('Exponential decrease schedule of threshold: step=%f',lambda);
	else	 	
		lambda=(deltamax-stopcriterion)/(itermax-1);	 % Slope of the linear decrease. 
		options.lambdasched = sprintf('Linear decrease schedule of threshold: step=%f',lambda);
    end
    
    h = waitbar(0,'MCA in progress: Please wait...');
	  nbpr=ceil(sqrt(numberofdicts+2));
	  subplot(nbpr,nbpr,1);imagesc(imgpad(1:N,1:M));axis image;rmaxis;drawnow;
	  subplot(nbpr,nbpr,2);imagesc(sum(part(1:N,1:M,:),3));axis image;rmaxis;title('\Sigma_i Part_i');drawnow;
	  for np=1:numberofdicts
	    subplot(nbpr,nbpr,np+2);imagesc(part(1:N,1:M,np));axis image;rmaxis;title(sprintf('Part_%d',np));drawnow;
	  end
    
%Start the modified Block Relaxation Algorithm
for iter = 0:itermax-1
    %Calculate the residual image.
    residual = imgpad-maskpad.*sum(part,3);
    
    %Cycle over dictrionaries.
    %for nb=1:numberoficts
    for nb=1:numberofdicts
        %Update Parta assuming other parts fixed.
        %Solve for Parta the marginal penalized minimization problem (Hard thresholding, 1_1->Soft)
        NAME = NthList(dict,nb);
        PAR1 = NthList(pars1,nb);
        PAR2 = NthList(pars2,nb);
        PAR3 = NthList(pars3,nb);
        Ra = part(:,:,nb)+residual;
        coeffa = FastLA2(Ra,NAME,PAR1,PAR2,PAR3);
        coeffa = eval([thdtype 'ThreshStruct(coeffa,delta,NAME);']);
        part(:,:,nb) = FastLS2(coeffa,NAME,PAR1,PAR2,PAR3);
        
        if (nb == comptv) & gamma ~= 0
            part(:,:,nb) = TVCorrection(part(:,:,nb),gamma);
        end
    end
    
    delta = delta-lambda;
    
    waitbar((iter+1)/itermax,h);
	    subplot(nbpr,nbpr,1);imagesc(imgpad(1:N,1:M));axis image;rmaxis;drawnow;
	    subplot(nbpr,nbpr,2);imagesc(sum(part(1:N,1:M,:),3));axis image;rmaxis;title('\Sigma_i Part_i');drawnow;
	    for np=1:numberofdicts
	    	subplot(nbpr,nbpr,np+2);imagesc(part(1:N,1:M,np));axis image;rmaxis;title(sprintf('Part_%d',np));drawnow;
	    end
    
end

part = part(1:N,1:M,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function C = HardThreshStruct(C,lambda,nameofdict)
global E

nbdicts = length(C);

for nb=1:nbdicts
 coeffs = C{nb};
 scaleindex = length(coeffs);
 if strcmp(nameofdict,'CURVWRAP')
   for j = 2:scaleindex
     for w = 1:length(coeffs(j).coeff)
       coeffs(j).coeff{w} = coeffs(j).coeff{w}.* (abs(coeffs(j).coeff{w}) > lambda*E{j}{w});
     end
   end
 else
   for j = 2:scaleindex
  	  coeffs(j).coeff = coeffs(j).coeff .* (abs(coeffs(j).coeff) > lambda);
   end
 end
 C{nb} = coeffs;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function C = SoftThreshStruct(C,lambda,nameofdict)
global E

nbdicts = length(C);

for nb=1:nbdicts
 coeffs = C{nb};
 scaleindex = length(coeffs);
 if strcmp(nameofdict,'CURVWRAP')
   for j = 2:scaleindex
     for w = 1:length(coeffs(j).coeff)
       coeffs(j).coeff{w} = sign(coeffs(j).coeff{w}) .* max(abs(coeffs(j).coeff{w}) - lambda*E{j}{w},0);
     end
   end
 else
   for j = 2:scaleindex
  	  coeffs(j).coeff = sign(coeffs(j).coeff) .* max(abs(coeffs(j).coeff) - lambda,0);
   end
 end
 C{nb} = coeffs;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function delta = StartingPoint(C,dict)
global E

nbdicts = length(C);

for nb=1:nbdicts
 tmp = [];
 coeffs = C{nb};
 scaleindex = length(coeffs);
 
 % If it is curvelet basis by the wrapping algorithm, then compute the L2 norm of the basis elements
 if strcmp(NthList(dict,nb),'CURVWRAP')
   computeL2norm(coeffs);
   for j = 2:scaleindex
     for w = 1:length(coeffs(j).coeff)
       wedge = coeffs(j).coeff{w}/E{j}{w};
       tmp = [tmp;wedge(:)];
     end
   end
 else
   for j = 2:scaleindex
  	  tmp = [tmp;coeffs(j).coeff(:)];
   end
 end
 buf(nb)=max(abs(tmp(:)));
end

%
buf=flipud(sort(buf(:),1))';
if nbdicts>1 delta=buf(2);
else	     delta=buf(1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = TVCorrection(x,gamma)
% Total variation implemented using the approximate (exact in 1D) equivalence between the TV norm and the l_1 norm of the Haar (heaviside) coefficients.

[n,J] = quadlength(x);

%qmf = MakeONFilter('Haar');

W = ndwt2(x,1,'Haar');

% [A,H,V,D] = swt2(x,1,'Haar');
% 
% H = SoftThresh(H,gamma);
% V = SoftThresh(V,gamma);
% D = SoftThresh(D,gamma);
% 
% y = iswt2(A,H,V,D,'Haar');

W.dec{2} = SoftThresh(W.dec{2},gamma);
W.dec{3} = SoftThresh(W.dec{3},gamma);
W.dec{4} = SoftThresh(W.dec{4},gamma);

y = indwt2(W);

% [ll,wc,L] = mrdwt(x,qmf,1);
% 
% wc = SoftThresh(wc,gamma);
% 
% y = mirdwt(ll,wc,qmf,1);

%y = tv(x,10);

