function [part,options]=MCA2_SIMPLE_LINELIKENESS(img,dict,pars1,pars2,pars3,itermax,gamma,comptv,expdecrease,stop,mask,sigma)
%Morphological Component Analysis of a 2D image using highly redundant
%dictrionaries and sparsity promoting penalties. This function is
%simplified from the MCALAB function 'MCA2_Bcr'.
%Usage:
%   part = MCA2_SIMPLE(img,dict,pars1,pars2,pars3,itermax,gamma,expdecrease,stop,mask)
%Inputs:
%   img:        2D image matrix, n*n, if n ~= 2^J, use zero-padding and crop
%   at the end.
%   dict:   Names of dictionaries for each part.
%   parsi:  Parameters of dictionary.
%   itermax:    Number of relaxation iterations.
%   gamma:  TV regularization parameter.
%   comptv: Component to which TV regularization is applied.
%   expdecrease:    Exponential/Linear decrease of the regularization
%   parameter.
%   stop:   Stop criterion, the algorithm stops when lambda <= stop*sigma
%   (typically k=3), sigma is the noise WGN std.
%Outputs:
%   parti:  Estimated i-th semantic component (n*n image)
%   options:    Structure containing all options and parameters of the alogrithm, including default choice.

%When using the dictionary function findline(I,t,n) and findnonline(I,t,n), different
%inital parameters t are assigned for different images to make the
%decomposition more accurate, we listed the values of t as follows for the
%users' convenience:
%
%   test_decomp_1:t=0.6;
%   test_decomp_2:t=0.65;
%   test_decomp_3:t=0.65;
%   test_decomp_4:t=0.6;
%   test_decomp_5:t=0.65;
%   test_decomp_6:t=0.65;
%   test_decomp_7:t=0.55;
%   test_decomp_8:t=0.57;
%   test_decomp_9:t=0.6;
%   test_decomp_10:t=0.55;
%   test_decomp_11:t=0.55;
%   test_decomp_12:t=0.55;
%   test_decomp_13:t=0.55;
%   test_decomp_14:t=0.6;
%   test_decomp_15:t=0.6;
%   test_decomp_16:t=0.6;
%   realworld_bream:t=0.9;
%   realworld_fan:t=0.9;
%   realworld_grain:t=0.9;
%   realworld_sharp1:t=0.9;
%   realworld_shell:t=0.9;
%   realworld_tiger:t=0.9;
%   realworld_toad:t=0.9;
%   realworld_tree1:t=0.9;
%   realworld_tree2:t=0.9;
%   realwrold_zibra:t=0.9;

% Initializations.
global E;
	[N,M]  	      = size(img);
	n      	      = 2^(nextpow2(max(N,M)));
	imgpad 	      = zeros(n,n);
	maskpad	      = ones(n,n);
	imgpad(1:N,1:M)  = img; %(img-min(min(img)))/(max(max(img))-min(min(img)));

    [n,J]  	      = quadlength(imgpad);
	thdtype       = MakeList('Smooth','Enhance');
	%options.thdtype = ['Threshold type: ' thdtype];

% Dictionary metadata.
	numberofdicts = LengthList(dict);
	options.nbcomp= sprintf('Number of morphological components: %d',numberofdicts);
	options.dict  = ['Transforms: [ ' dict ']'];
	str = 'Parameter 1 of transforms: [ ';
	for nb=1:numberofdicts, str = [str num2str(NthList(pars1,nb)) ' : ']; end
	options.pars1 = [str ']'];
	str = 'Parameter 2 of transforms: [ ';
	for nb=1:numberofdicts, str = [str num2str(NthList(pars2,nb)) ' : ']; end
	options.pars2 = [str ']'];	
	str = 'Parameter 3 of transforms: [ ';
	for nb=1:numberofdicts, str = [str num2str(NthList(pars3,nb)) ' : ']; end
	options.pars3 = [str ']'];
	part	      = zeros(n,n,numberofdicts);
    
% To estimate the WGN standard deviation using the MAD.
% The qmf is quite arbitrary, sufficiently regular to approach a good band-pass.
	if ~exist('sigma','var') | isempty(sigma),
	  qmf=MakeONFilter('Daubechies',4);
	  wc = FWT2_PO(imgpad,J-1,qmf);
	  hh = wc(n/2+1:n/2+floor(N/2),n/2+1:n/2+floor(M/2));hh = hh(:);
	  tmp   = maskpad(n/2+1:n/2+floor(N/2),n/2+1:n/2+floor(M/2));tmp = tmp(:);
	  sigma = MAD(hh(find(tmp)));
	  options.sigma = sprintf('Initial sigma estimated from data: %f',sigma);
	else
	  options.sigma = sprintf('Initial sigma fixed by the user: %f',sigma);
	end
	%stopcriterion = stop*sigma;
    stopcriterion = 0.65;
	options.stopcriterion = sprintf('Stopping threshold: %f x sigma=%f',stopcriterion);
	
% First pass: coeffs of the original image in each dictionary.
	coeff = FastLA2(imgpad,dict,pars1,pars2,pars3);

% Calculate the starting thd, which is the minimum of maximal coefficients 
% of the image in each dictionary.
	%deltamax=StartingPoint(coeff,dict);
    deltamax = 0.95;
	delta=deltamax;
	options.lambdamax   = sprintf('Starting threshold: %f',deltamax);
	if expdecrease	
		lambda=(deltamax/stopcriterion)^(1/(1-itermax)); % Exponential decrease.
		options.lambdasched = sprintf('Exponential decrease schedule of threshold: step=%f',lambda);
	else	 	
		lambda=(deltamax-stopcriterion)/(itermax-1);	 % Slope of the linear decrease. 
		options.lambdasched = sprintf('Linear decrease schedule of threshold: step=%f',lambda);
    end
    
    h = waitbar(0,'MCA in progress: Please wait...');
	  nbpr=ceil(sqrt(numberofdicts+2));
	  subplot(nbpr,nbpr,1);imagesc(imgpad(1:N,1:M));axis image;rmaxis;drawnow;
	  subplot(nbpr,nbpr,2);imagesc(sum(part(1:N,1:M,:),3));axis image;rmaxis;title('\Sigma_i Part_i');drawnow;
	  for np=1:numberofdicts
	    subplot(nbpr,nbpr,np+2);imagesc(part(1:N,1:M,np));axis image;rmaxis;title(sprintf('Part_%d',np));drawnow;
	  end
    
%Start the modified Block Relaxation Algorithm
for iter = 0:itermax-1
    %Calculate the residual image.
    residual = imgpad-part(:,:,1)-part(:,:,2);
%     residual = imgpad-maskpad.*sum(part,3)/2;
%     residual = (residual-min(min(residual)))/(max(max(residual))-min(min(residual)));
    
    %Cycle over dictrionaries.
    %for nb=1:numberoficts
    for nb=1:numberofdicts
        %Update Parta assuming other parts fixed.
        %Solve for Parta the marginal penalized minimization problem (Hard thresholding, 1_1->Soft)
        NAME = NthList(dict,nb);
        PAR1 = NthList(pars1,nb);
        PAR2 = NthList(pars2,nb);
        PAR3 = NthList(pars3,nb);
        Ra1 = part(:,:,1)+residual;
        Ra2 = part(:,:,2)+residual;
        
%         Ra = (Ra-min(min(Ra)))/(max(max(Ra))-min(min(Ra)));
%         coeffa = FastLA2(Ra,NAME,PAR1,PAR2,PAR3);
%         THRESH = NthList(thdtype,nb);
%         coeffa = eval([THRESH 'ThreshStruct(coeffa,delta);']);
%         part(:,:,nb) = FastLS2(coeffa,NAME,PAR1,PAR2,PAR3);
        if nb == 1
            %This is better for the synthesis image
%             se = strel('square',8);
%             temp1 = imerode(Ra,se);
%             part(:,:,nb) = imdilate(temp1,se);
%             part(:,:,nb) = cartoon(Ra);
    %Bilateral filter is better for the actual image
            part(:,:,nb) = findline(Ra1,0.6,1);%for the actual one texturemixture2.png, [1,1];

%                           part(:,:,nb) = imadjust(anisodiff2D_1(im2uint8(Ra),10,10)/255);
%             part(:,:,nb) = TVCorrection(part(:,:,nb),gamma);
        else
            if iter == 0
              part(:,:,nb) = findnonline(imgpad,0.6,1);
            else
              part(:,:,nb) = part(:,:,nb) + findnonline(imgpad-Ra2,0.6,1);
            end
%                 part(:,:,nb) = fftfilt(Ra,1-0.01.*iter);
%               part(:,:,nb) = Ra - anisodiff2D(im2uint8(Ra),50,1/8,10,1)/255;

            
        end
        
%         part(:,:,nb) = (part(:,:,nb)-min(min(part(:,:,nb))))/(max(max(part(:,:,nb)))-min(min(part(:,:,nb))));
               
        if (nb == comptv) & gamma ~= 0
            part(:,:,nb) = TVCorrection(part(:,:,nb),gamma);
        end
    end
    
    delta = delta-lambda;
    
    waitbar((iter+1)/itermax,h);
	    subplot(nbpr,nbpr,1);imagesc(imgpad(1:N,1:M));axis image;rmaxis;drawnow;
	    subplot(nbpr,nbpr,2);imagesc(sum(part(1:N,1:M,:),3));axis image;rmaxis;title('\Sigma_i Part_i');drawnow;
	    for np=1:numberofdicts
	    	subplot(nbpr,nbpr,np+2);imagesc(part(1:N,1:M,np));axis image;rmaxis;title(sprintf('Part_%d',np));drawnow;
	    end
    
end

part = part(1:N,1:M,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function C = FineEnhancement(I3)

[a h v d] = dwt2(I3,'sym8');
a1 = a;
h1 = h;
v1 = v;
d1 = d;
[m n] = size(a);

h_max = max(max(abs(h)));
v_max = max(max(abs(v)));
d_max = max(max(abs(d)));

for x = 1 : m
    for y = 1 : n
        if abs(h(x,y)) >= 0.5*h_max || abs(v(x,y)) >= 0.5*v_max || abs(d(x,y)) >= 0.5*d_max
%             a1(x,y) = 0;
            h1(x,y) = 0;
            v1(x,y) = 0;
            d1(x,y) = 0;
        end
    end
end

C = idwt2(a1,h1,v1,d1,'sym8');



function C = SmoothThreshStruct(C,lambda)


global E

nbdicts = length(C);

for nb = 1:nbdicts
    coeffs = C{nb};
    scaleindex = length(coeffs);
    for j = 2 : scaleindex
        
        sub_coef_h = coeffs(j).coeff{1,2};
        sub_coef_v = coeffs(j).coeff{2,1};
        sub_coef_d = coeffs(j).coeff{2,2};
        
        for i = 1 : 3
            sub_coef_h(:,:,i) = SubbandThresholding(sub_coef_h(:,:,i));
            sub_coef_v(:,:,i) = SubbandThresholding(sub_coef_v(:,:,i));
            sub_coef_d(:,:,i) = SubbandThresholding(sub_coef_d(:,:,i));
        end
    end
    
    coeffs(j).coeff{1,2} = sub_coef_h;
    coeffs(j).coeff{2,1} = sub_coef_v;
    coeffs(j).coeff{2,2} = sub_coef_d;
    
    C{nb} = coeffs;
        
end


function C = EnhanceThreshStruct(C,lambda)


global E

nbdicts = length(C);

for nb = 1:nbdicts
    coeffs = C{nb};
    scaleindex = length(coeffs);
    for j = 2 : scaleindex
        
        sub_coef_l = coeffs(j).coeff{1,1};
        sub_coef_h = coeffs(j).coeff{1,2};
        sub_coef_v = coeffs(j).coeff{2,1};
        sub_coef_d = coeffs(j).coeff{2,2};
        
        
        for i = 1 : 3
            
            l = sub_coef_l(:,:,i);
            h = sub_coef_h(:,:,i);
            v = sub_coef_v(:,:,i);
            d = sub_coef_d(:,:,i);
            
            
            ext_h = padarray(h,[2 2],0);
            ext_v = padarray(v,[2 2],0);
            ext_d = padarray(d,[2 2],0);
            
            [nRow,nCol] = size(l);
            t_l = zeros(nRow,nCol);
            t_h = t_l;
            t_v = t_l;
            t_d = t_l;
            
            thresh_h = lambda.*max(max(abs(h(:))));
            thresh_v = lambda.*max(max(abs(v(:))));
            thresh_d = lambda.*max(max(abs(d(:))));
            
            for a = 1+2:nRow+2
                for b = 1+2:nCol+2
                    if abs(h(a-2,b-2))>=thresh_h
                        factor_h = abs(h(a-2,b-2))/thresh_h;
                    else
                        factor_h = 0;
                    end
                    
                    if abs(v(a-2,b-2))>=thresh_v
                        factor_v = abs(v(a-2,b-2))/thresh_v;
                    else
                        factor_v = 0;
                    end
                    
                    if abs(d(a-2,b-2))>=thresh_d
                        factor_d = abs(d(a-2,b-2))/thresh_d;
                    else
                        factor_d = 0;
                    end
                    
                    if factor_h == 0 && factor_v == 0 && factor_d == 0;
                        factor_l = 0;
                    else
                        factor_l = 1;
                    end

%                     factor_h = max(abs(h(a-2,b-2))/thresh_h,1);
%                     factor_v = max(abs(v(a-2,b-2))/thresh_v,1);
%                     factor_d = max(abs(d(a-2,b-2))/thresh_d,1);
                    
                    t_l(a-2,b-2) = factor_l*l(a-2,b-2);
                    t_h(a-2,b-2) = factor_h*h(a-2,b-2);
                    t_v(a-2,b-2) = factor_v*v(a-2,b-2);
                    t_d(a-2,b-2) = factor_d*d(a-2,b-2);
                    
%                     t_h(a-2:a+2,b-2:b+2) = factor_h.*ext_h(a-2:a+2,b-2:b+2);
%                     t_v(a-2:a+2,b-2:b+2) = factor_v.*ext_v(a-2:a+2,b-2:b+2);
%                     t_d(a-2:a+2,b-2:b+2) = factor_d.*ext_d(a-2:a+2,b-2:b+2);
                end
            end
            
            sub_coef_l(:,:,i) = t_l;
            sub_coef_h(:,:,i) = t_h;
            sub_coef_v(:,:,i) = t_v;
            sub_coef_d(:,:,i) = t_d;
            
%             sub_coef_h(:,:,i) = t_h(1+2:nRow+2,1+2:nCol+2);
%             sub_coef_v(:,:,i) = t_v(1+2:nRow+2,1+2:nCol+2);
%             sub_coef_d(:,:,i) = t_d(1+2:nRow+2,1+2:nCol+2);
            
        end
    end
    
    coeffs(j).coeff{1,1} = sub_coef_l;
    coeffs(j).coeff{1,2} = sub_coef_h;
    coeffs(j).coeff{2,1} = sub_coef_v;
    coeffs(j).coeff{2,2} = sub_coef_d;
    
    C{nb} = coeffs;
end



function C = HardThreshStruct(C,lambda,nameofdict)
global E

nbdicts = length(C);

for nb=1:nbdicts
 coeffs = C{nb};
 scaleindex = length(coeffs);
 if strcmp(nameofdict,'CURVWRAP')
   for j = 2:scaleindex
     for w = 1:length(coeffs(j).coeff)
       coeffs(j).coeff{w} = coeffs(j).coeff{w}.* (abs(coeffs(j).coeff{w}) > lambda*E{j}{w});
     end
   end
 else
   for j = 2:scaleindex
  	  coeffs(j).coeff = coeffs(j).coeff .* (abs(coeffs(j).coeff) > lambda);
   end
 end
 C{nb} = coeffs;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function C = SoftThreshStruct(C,lambda,nameofdict)
global E

nbdicts = length(C);

for nb=1:nbdicts
 coeffs = C{nb};
 scaleindex = length(coeffs);
 if strcmp(nameofdict,'CURVWRAP')
   for j = 2:scaleindex
     for w = 1:length(coeffs(j).coeff)
       coeffs(j).coeff{w} = sign(coeffs(j).coeff{w}) .* max(abs(coeffs(j).coeff{w}) - lambda*E{j}{w},0);
     end
   end
 else
   for j = 2:scaleindex
  	  coeffs(j).coeff = sign(coeffs(j).coeff) .* max(abs(coeffs(j).coeff) - lambda,0);
   end
 end
 C{nb} = coeffs;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function delta = StartingPoint(C,dict)
global E

nbdicts = length(C);

for nb=1:nbdicts
 tmp = [];
 coeffs = C{nb};
 scaleindex = length(coeffs);
 
 % If it is curvelet basis by the wrapping algorithm, then compute the L2 norm of the basis elements
 if strcmp(NthList(dict,nb),'CURVWRAP')
   computeL2norm(coeffs);
   for j = 2:scaleindex
     for w = 1:length(coeffs(j).coeff)
       wedge = coeffs(j).coeff{w}/E{j}{w};
       tmp = [tmp;wedge(:)];
     end
   end
 else
   for j = 2:scaleindex
  	  tmp = [tmp;coeffs(j).coeff(:)];
   end
 end
 buf(nb)=max(abs(tmp(:)));
end

%
buf=flipud(sort(buf(:),1))';
if nbdicts>1 delta=buf(2);
else	     delta=buf(1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y = TVCorrection(x,gamma)
% Total variation implemented using the approximate (exact in 1D) equivalence between the TV norm and the l_1 norm of the Haar (heaviside) coefficients.

[n,J] = quadlength(x);

%qmf = MakeONFilter('Haar');

W = ndwt2(x,1,'Haar');

% [A,H,V,D] = swt2(x,1,'Haar');
% 
% H = SoftThresh(H,gamma);
% V = SoftThresh(V,gamma);
% D = SoftThresh(D,gamma);
% 
% y = iswt2(A,H,V,D,'Haar');

W.dec{2} = SoftThresh(W.dec{2},gamma);
W.dec{3} = SoftThresh(W.dec{3},gamma);
W.dec{4} = SoftThresh(W.dec{4},gamma);

y = indwt2(W);

% [ll,wc,L] = mrdwt(x,qmf,1);
% 
% wc = SoftThresh(wc,gamma);
% 
% y = mirdwt(ll,wc,qmf,1);

%y = tv(x,10);

