function diff_im = anisoshrink2D(im, num_iter, delta_t,kappa)
%ANISODIFF2D Conventional anisotropic diffusion
%   DIFF_IM = ANISODIFF2D(IM, NUM_ITER, DELTA_T, KAPPA, OPTION) perfoms 
%   conventional anisotropic diffusion (Perona & Malik) upon a gray scale
%   image. A 2D network structure of 8 neighboring nodes is considered for 
%   diffusion conduction.
% 
%       ARGUMENT DESCRIPTION:
%               IM       - gray scale image (MxN).
%               NUM_ITER - number of iterations. 
%               DELTA_T  - integration constant (0 <= delta_t <= 1/7).
%                          Usually, due to numerical stability this 
%                          parameter is set to its maximum value.
%               KAPPA    - gradient modulus threshold that controls the conduction.
%               OPTION   - conduction coefficient functions proposed by Perona & Malik:
%                          1 - c(x,y,t) = exp(-(nablaI/kappa).^2),
%                              privileges high-contrast edges over low-contrast ones. 
%                          2 - c(x,y,t) = 1./(1 + (nablaI/kappa).^2),
%                              privileges wide regions over smaller ones. 
% 
%       OUTPUT DESCRIPTION:
%                DIFF_IM - (diffused) image with the largest scale-space parameter.
% 
%   Example
%   -------------
%   s = phantom(512) + randn(512);
%   num_iter = 15;
%   delta_t = 1/7;
%   kappa = 30;
%   option = 2;
%   ad = anisodiff2D(s,num_iter,delta_t,kappa,option);
%   figure, subplot 121, imshow(s,[]), subplot 122, imshow(ad,[])
% 
% See also anisodiff1D, anisodiff3D.

% References: 
%   P. Perona and J. Malik. 
%   Scale-Space and Edge Detection Using Anisotropic Diffusion.
%   IEEE Transactions on Pattern Analysis and Machine Intelligence, 
%   12(7):629-639, July 1990.
% 
%   G. Grieg, O. Kubler, R. Kikinis, and F. A. Jolesz.
%   Nonlinear Anisotropic Filtering of MRI Data.
%   IEEE Transactions on Medical Imaging,
%   11(2):221-232, June 1992.
% 
%   MATLAB implementation based on Peter Kovesi's anisodiff(.):
%   P. D. Kovesi. MATLAB and Octave Functions for Computer Vision and Image Processing.
%   School of Computer Science & Software Engineering,
%   The University of Western Australia. Available from:
%   <http://www.csse.uwa.edu.au/~pk/research/matlabfns/>.
% 
% Credits:
% Daniel Simoes Lopes
% ICIST
% Instituto Superior Tecnico - Universidade Tecnica de Lisboa
% danlopes (at) civil ist utl pt
% http://www.civil.ist.utl.pt/~danlopes
%
% May 2007 original version.

% Convert input image to double.
im = double(im);

% PDE (partial differential equation) initial condition.
diff_im = im;

% Center pixel distances.
dx = 1;
dy = 1;
dd = sqrt(2);

[r c] = size(im);

% 2D convolution masks - finite differences.
hN = [0 -1 0; 0 1 0; 0 0 0];
hS = [0 0 0; 0 1 0; 0 -1 0];
hE = [0 0 0; 0 1 -1; 0 0 0];
hW = [0 0 0; -1 1 0; 0 0 0];
hNE = [0 0 -1; 0 1 0; 0 0 0];
hSE = [0 0 0; 0 1 0; 0 0 -1];
hSW = [0 0 0; 0 1 0; -1 0 0];
hNW = [-1 0 0; 0 1 0; 0 0 0];
sigma = zeros(size(im));

% Anisotropic diffusion.
for t = 1:num_iter
%     i_t = padarray(diff_im,[1 1],'symmetric');
%     for x = 2:r+1
%         for y = 2:c+1
%             temp = i_t(x-1:x+1,y-1:y+1);
%             sigma(x-1,y-1) = var(temp(:));
%         end
%     end
%     sigma_max = max(sigma(:));
%     sigma_min = min(sigma(:));
%     
%     sigma_t = 1 + (sigma-sigma_min)/(sigma_max-sigma_min)*254;
        % Finite differences. [imfilter(.,.,'conv') can be replaced by conv2(.,.,'same')]
        nablaN = imfilter(diff_im,hN,'symmetric');
        nablaS = imfilter(diff_im,hS,'symmetric');   
        nablaW = imfilter(diff_im,hW,'symmetric');
        nablaE = imfilter(diff_im,hE,'symmetric');   
        nablaNE = imfilter(diff_im,hNE,'symmetric');
        nablaSE = imfilter(diff_im,hSE,'symmetric');   
        nablaSW = imfilter(diff_im,hSW,'symmetric');
        nablaNW = imfilter(diff_im,hNW,'symmetric');
        
        cN = 1-exp(-(nablaN/kappa).^2);
        cS = 1-exp(-(nablaS/kappa).^2);
        cW = 1-exp(-(nablaW/kappa).^2);
        cE = 1-exp(-(nablaE/kappa).^2);
        cNE = 1+exp(-(nablaNE/kappa).^2);
        cSE = 1+exp(-(nablaSE/kappa).^2);
        cSW = 1+exp(-(nablaSW/kappa).^2);
        cNW = 1+exp(-(nablaNW/kappa).^2);
        
%         Avg = 1/4.*(abs(nablaN)+abs(nablaS)+abs(nablaW)+abs(nablaE));
%         cAvg = exp((Avg/kappa).^2);
        diff_im = diff_im + ...
                  delta_t*((1/(dy^2))*cN.*abs(nablaN) + (1/(dy^2))*cS.*abs(nablaS) + ...
                  (1/(dx^2))*cW.*abs(nablaW) + (1/(dx^2))*cE.*abs(nablaE));
        
%         1/2*cNE.*nablaNE+1/2*cSE.*nablaSE+1/2*cSW.*nablaSW+1/2*cNW.*nablaNW);
%         compare = zeros(r,c,8);
%         
%         compare(:,:,1) = nablaN;
%         compare(:,:,2) = nablaS;
%         compare(:,:,3) = nablaW;
%         compare(:,:,4) = nablaE;
%         compare(:,:,5) = nablaNE;
%         compare(:,:,6) = nablaSE;
%         compare(:,:,7) = nablaSW;
%         compare(:,:,8) = nablaNW;
%         
%         delta_avg = max(compare,[],3);
        
%         delta_avg(abs(delta_avg)<T) = 0;
        
%         cD = 1- exp(-(delta_avg/T).^2);
        
%         diff_im = diff_im - delta_avg;
        
%         nablaN(nablaN<0) = 0;
%         nablaS(nablaS<0) = 0;
%         nablaW(nablaW<0) = 0;
%         nablaE(nablaE<0) = 0;
%         nablaNE(nablaNE<0) = 0;
%         nablaSE(nablaSE<0) = 0;
%         nablaSW(nablaSW<0) = 0;
%         nablaNW(nablaNW<0) = 0;
%         
%         % Diffusion function.
%         if option == 1
%             cN = 1-exp(-(nablaN/kappa));
%             cS = 1-exp(-(nablaS/kappa));
%             cW = 1-exp(-(nablaW/kappa));
%             cE =1- exp(-(nablaE/kappa));
%             cNE = 1-exp(-(nablaNE/kappa));
%             cSE = 1-exp(-(nablaSE/kappa));
%             cSW = 1-exp(-(nablaSW/kappa));
%             cNW = 1-exp(-(nablaNW/kappa));
%         elseif option == 2
%             cN = 1./(1 + (nablaN.*sigma_t/kappa).^2);
%             cS = 1./(1 + (nablaS.*sigma_t/kappa).^2);
%             cW = 1./(1 + (nablaW.*sigma_t/kappa).^2);
%             cE = 1./(1 + (nablaE.*sigma_t/kappa).^2);
%             cNE = 1./(1 + (nablaNE.*sigma_t/kappa).^2);
%             cSE = 1./(1 + (nablaSE.*sigma_t/kappa).^2);
%             cSW = 1./(1 + (nablaSW.*sigma_t/kappa).^2);
%             cNW = 1./(1 + (nablaNW.*sigma_t/kappa).^2);
%         end
% 
%         % Discrete PDE solution.
%         diff_im = diff_im + ...
%                   delta_t*(...
%                   (1/(dy^2))*cN.*abs(nablaN) + (1/(dy^2))*cS.*abs(nablaS) + ...
%                   (1/(dx^2))*cW.*abs(nablaW) + (1/(dx^2))*cE.*abs(nablaE));
%                   (1/(dd^2))*cNE.*abs(nablaNE) + (1/(dd^2))*cSE.*abs(nablaSE) + ...
%                   (1/(dd^2))*cSW.*abs(nablaSW) + (1/(dd^2))*cNW.*abs(nablaNW));
%            
        % Iteration warning.
%         diff_im = diff_im/255;
%         fprintf('\rIteration %d\n',t);
end

diff_im = diff_im/(max(diff_im(:)));